@@ -1,15 +1,25 @@
 import itertools, re, sys
 
 from django.conf import urls
-from django.core import exceptions
+from django.core import exceptions, urlresolvers
 from django.db.models import base as models_base
-from django.db.models.sql import constants
 from django.utils import datastructures
 
+try:
+    # Django 1.5+
+    from django.db.models import constants
+except ImportError:
+    # Before Django 1.5
+    from django.db.models.sql import constants
+
 from tastypie import bundle as tastypie_bundle, exceptions as tastypie_exceptions, fields as tastypie_fields, http, resources, utils
 
 import mongoengine
 from mongoengine import fields as mongoengine_fields, queryset
+try:
+    from mongoengine.queryset import tranform as mongoengine_tranform
+except ImportError:
+    mongoengine_tranform = None
 
 from tastypie_mongoengine import fields
 
@@ -17,12 +27,13 @@ from tastypie_mongoengine import fields
 # We use a mock Query object to provide the same interface and return query terms by MongoEngine. 
 # MongoEngine code might not expose these query terms, so we fallback to hard-coded values.
 
-QUERY_TERMS_ALL = getattr(queryset, 'QUERY_TERMS_ALL', ('ne', 'gt', 'gte', 'lt', 'lte', 'in', 'nin', 'mod', 'all', 'size', 'exists', 'not', 'within_distance', 'within_spherical_distance', 'within_box', 'within_polygon', 'near', 'near_sphere','contains', 'icontains', 'startswith', 'istartswith', 'endswith', 'iendswith', 'exact', 'iexact', 'match'))
+QUERY_TERMS_ALL = getattr(mongoengine_tranform, 'MATCH_OPERATORS', ('ne', 'gt', 'gte', 'lt', 'lte', 'in', 'nin', 'mod', 'all', 'size', 'exists', 'not', 'within_distance', 'within_spherical_distance', 'within_box', 'within_polygon', 'near', 'near_sphere','contains', 'icontains', 'startswith', 'istartswith', 'endswith', 'iendswith', 'exact', 'iexact', 'match'))
 
 class Query(object):
     query_terms = dict([(query_term, None) for query_term in QUERY_TERMS_ALL])
 
-queryset.QuerySet.query = Query()
+if not hasattr(queryset.QuerySet, 'query'):
+    queryset.QuerySet.query = Query()
 
 CONTENT_TYPE_RE = re.compile('.*; type=([\w\d-]+);?')
 
@@ -30,6 +41,9 @@ class NOT_HYDRATED:
     pass
 
 class ListQuerySet(datastructures.SortedDict):
+    # Workaround for https://github.com/toastdriven/django-tastypie/pull/670
+    query = Query()
+
     def _process_filter_value(self, value):
         # Sometimes value is passed as a list of one value
         # (if filter was converted from QueryDict, for example)
@@ -166,6 +180,10 @@ class MongoEngineModelDeclarativeMetaclass(resources.ModelDeclarativeMetaclass):
             if hasattr(meta, 'object_class') and not hasattr(meta, 'queryset'):
                 if hasattr(meta.object_class, 'objects'):
                     setattr(meta, 'queryset', meta.object_class.objects.all())
+                elif issubclass(meta.object_class, mongoengine.EmbeddedDocument):
+                    # Workaround for https://github.com/toastdriven/django-tastypie/pull/670
+                    # We ignore queryset value later on, so we can set it here to empty one
+                    setattr(meta, 'queryset', ListQuerySet())
 
         new_class = super(resources.ModelDeclarativeMetaclass, self).__new__(self, name, bases, attrs)
         include_fields = getattr(new_class._meta, 'fields', [])
@@ -221,6 +239,19 @@ class MongoEngineModelDeclarativeMetaclass(resources.ModelDeclarativeMetaclass):
             else:
                 seen_types.add(type_map[typ]._meta.object_class)
 
+        if new_class._meta.object_class:
+            # In MongoEngine 0.7.6+ embedded documents do not have exceptions anymore,
+            # but this prevents are from reusing existing Tastypie code
+
+            exceptions_to_merge = [exc for exc in (queryset.DoesNotExist, queryset.MultipleObjectsReturned) if not hasattr(new_class._meta.object_class, exc.__name__)]
+            module = new_class._meta.object_class.__module__
+            for exc in exceptions_to_merge:
+                name = exc.__name__
+                parents = tuple(getattr(base, name) for base in new_class._meta.object_class._get_bases(bases) if hasattr(base, name)) or (exc,)
+                # Create new exception and set to new_class
+                exception = type(name, parents, {'__module__': module})
+                setattr(new_class._meta.object_class, name, exception)
+
         return new_class
 
 class MongoEngineResource(resources.ModelResource):
@@ -297,6 +328,9 @@ class MongoEngineResource(resources.ModelResource):
             self._meta.queryset = resource._meta.queryset
             self.base_fields = resource.base_fields.copy()
             self.fields = resource.fields.copy()
+            if getattr(self._meta, 'prefer_polymorphic_resource_uri', False):
+                if resource.get_resource_uri():
+                    self._meta.resource_name = resource._meta.resource_name
             if getattr(self._meta, 'include_resource_type', True):
                 self.base_fields['resource_type'] = base_fields['resource_type']
                 self.fields['resource_type'] = fields['resource_type']
@@ -374,17 +408,17 @@ class MongoEngineResource(resources.ModelResource):
 
         return self._get_type_from_class(type_map, bundle.obj.__class__)
 
-    def full_dehydrate(self, bundle):
+    def full_dehydrate(self, bundle, for_list=False):
         type_map = getattr(self._meta, 'polymorphic', {})
         if not type_map:
-            return super(MongoEngineResource, self).full_dehydrate(bundle)
+            return super(MongoEngineResource, self).full_dehydrate(bundle, for_list)
 
         # Optimization
         if self._meta.object_class is bundle.obj.__class__:
-            return super(MongoEngineResource, self).full_dehydrate(bundle)
+            return super(MongoEngineResource, self).full_dehydrate(bundle, for_list)
 
         resource = self._get_resource_from_class(type_map, bundle.obj.__class__)(self._meta.api_name)
-        return self._wrap_polymorphic(resource, lambda: super(MongoEngineResource, self).full_dehydrate(bundle))
+        return self._wrap_polymorphic(resource, lambda: super(MongoEngineResource, self).full_dehydrate(bundle, for_list))
 
     def full_hydrate(self, bundle):
         # When updating objects, we want to force only updates of the same type, and object
@@ -472,10 +506,10 @@ class MongoEngineResource(resources.ModelResource):
 
         return data
 
-    def obj_get(self, request=None, **kwargs):
+    def obj_get(self, bundle, **kwargs):
         # MongoEngine exceptions are separate from Django exceptions, we combine them here
         try:
-            return super(MongoEngineResource, self).obj_get(request, **kwargs)
+            return super(MongoEngineResource, self).obj_get(bundle=bundle, **kwargs)
         except self._meta.object_class.DoesNotExist, e:
             exp = models_base.subclass_exception('DoesNotExist', (self._meta.object_class.DoesNotExist, exceptions.ObjectDoesNotExist), self._meta.object_class.DoesNotExist.__module__)
             raise exp(*e.args)
@@ -492,45 +526,43 @@ class MongoEngineResource(resources.ModelResource):
             exp = models_base.subclass_exception('DoesNotExist', (queryset.DoesNotExist, exceptions.ObjectDoesNotExist), queryset.DoesNotExist.__module__)
             raise exp(*e.args)
 
-    def obj_create(self, bundle, request=None, **kwargs):
-        try:
-            self._reset_collection()
-            return super(MongoEngineResource, self).obj_create(bundle, request, **kwargs)
-        except mongoengine.ValidationError, e:
-            raise exceptions.ValidationError(e.message)
+    def obj_create(self, bundle, **kwargs):
+        self._reset_collection()
+        return super(MongoEngineResource, self).obj_create(bundle, **kwargs)
 
-    def obj_update(self, bundle, request=None, **kwargs):
-        try:
-            self._reset_collection()
+    # TODO: Use skip_errors?
+    def obj_update(self, bundle, skip_errors=False, **kwargs):
+        self._reset_collection()
 
-            if not bundle.obj or not getattr(bundle.obj, 'pk', None):
-                try:
-                    bundle.obj = self.obj_get(request, **kwargs)
-                except (queryset.DoesNotExist, exceptions.ObjectDoesNotExist):
-                    raise tastypie_exceptions.NotFound("A document instance matching the provided arguments could not be found.")
+        if not bundle.obj or not getattr(bundle.obj, 'pk', None):
+            try:
+                bundle.obj = self.obj_get(bundle=bundle, **kwargs)
+            except (queryset.DoesNotExist, exceptions.ObjectDoesNotExist):
+                raise tastypie_exceptions.NotFound("A document instance matching the provided arguments could not be found.")
 
-            bundle = self.full_hydrate(bundle)
+        self.authorized_update_detail(self.get_object_list(bundle.request), bundle)
+        bundle = self.full_hydrate(bundle)
+        return self.save(bundle, skip_errors=skip_errors)
 
-            self.save_related(bundle)
-
-            bundle.obj.save()
-
-            m2m_bundle = self.hydrate_m2m(bundle)
-            self.save_m2m(m2m_bundle)
-            return bundle
-        except mongoengine.ValidationError, e:
-            raise exceptions.ValidationError(e.message)
-
-    def obj_delete(self, request=None, **kwargs):
+    def obj_delete(self, bundle, **kwargs):
         self._reset_collection()
 
         # MongoEngine exceptions are separate from Django exceptions and Tastypie
         # expects Django exceptions, so we catch it here ourselves and raise NotFound
         try:
-            return super(MongoEngineResource, self).obj_delete(request, **kwargs)
+            return super(MongoEngineResource, self).obj_delete(bundle, **kwargs)
         except queryset.DoesNotExist:
             raise tastypie_exceptions.NotFound("A document instance matching the provided arguments could not be found.")
 
+    def create_identifier(self, obj):
+        return unicode(obj.pk)
+
+    def save(self, bundle, skip_errors=False):
+        try:
+            return super(MongoEngineResource, self).save(bundle, skip_errors)
+        except mongoengine.ValidationError, e:
+            raise exceptions.ValidationError(e.message)
+
     def save_m2m(self, bundle):
         # Our related documents are not stored in a queryset, but a list,
         # so we have to manually build a list, set it, and save
@@ -667,6 +699,30 @@ class MongoEngineResource(resources.ModelResource):
 
         return final_fields
 
+        raise IndexError("Embedded document with primary key '%s' not found." % pk)
+
+    def update_in_place(self, request, original_bundle, new_data):
+        """
+        Update the object in original_bundle in-place using new_data.
+        """
+
+        # TODO: Is this the place to use MongoDB atomic operations to update the document?
+
+        from tastypie.utils import dict_strip_unicode_keys
+        original_bundle.data.update(**dict_strip_unicode_keys(new_data))
+
+        # Now we've got a bundle with the new data sitting in it and we're
+        # we're basically in the same spot as a PUT request. So the rest of this
+        # function is cribbed from put_detail.
+        self.alter_deserialized_detail_data(request, original_bundle.data)
+        
+        # Removed request from kwargs, breaking obj_get filter, currently present
+        # in tastypie. See https://github.com/toastdriven/django-tastypie/issues/824.
+        kwargs = {
+            self._meta.detail_uri_name: self.get_bundle_detail_data(original_bundle),
+        }
+        return self.obj_update(bundle=original_bundle, **kwargs)
+
 class MongoEngineListResource(MongoEngineResource):
     """
     A MongoEngine resource used in conjunction with EmbeddedListField.
@@ -690,18 +746,19 @@ class MongoEngineListResource(MongoEngineResource):
                 if not current_pk:
                     self._meta.id_field = field_name
 
-    def _safe_get(self, request, **kwargs):
+    def _safe_get(self, bundle, **kwargs):
         filters = self.remove_api_resource_names(kwargs)
 
         try:
-            return self.parent.cached_obj_get(request=request, **filters)
+            return self.parent.cached_obj_get(bundle=bundle, **filters)
         except (queryset.DoesNotExist, exceptions.ObjectDoesNotExist):
             raise tastypie_exceptions.ImmediateHttpResponse(response=http.HttpNotFound())
 
     def dispatch(self, request_type, request, **kwargs):
         subresource_pk = kwargs.pop('subresource_pk', None)
 
-        self.instance = self._safe_get(request, **kwargs)
+        bundle = self.build_bundle(request=request)
+        self.instance = self._safe_get(bundle, **kwargs)
 
         # We use subresource pk as pk from now on
         kwargs['pk'] = subresource_pk
@@ -740,7 +797,7 @@ class MongoEngineListResource(MongoEngineResource):
 
             return ListQuerySet([(unicode(index), add_index(index, obj)) for index, obj in enumerate(getattr(self.instance, self.attribute))])
 
-    def obj_create(self, bundle, request=None, **kwargs):
+    def obj_create(self, bundle, **kwargs):
         try:
             bundle.obj = self._meta.object_class()
 
@@ -769,11 +826,19 @@ class MongoEngineListResource(MongoEngineResource):
         except mongoengine.ValidationError, e:
             raise exceptions.ValidationError(e.message)
 
-    def obj_update(self, bundle, request=None, **kwargs):
+    def find_embedded_document(self, objects, pk_field, pk):
+        # TODO: Would it be faster to traverse in reversed direction? Because probably last elements are fetched more often in practice?
+        # TODO: Should we cache information about mappings between IDs and elements?
+        for i, obj in enumerate(objects):
+            if getattr(obj, pk_field) == pk:
+                return i
+
+    # TODO: Use skip_errors?
+    def obj_update(self, bundle, skip_errors=False, **kwargs):
         try:
             if not bundle.obj or not getattr(bundle.obj, 'pk', None):
                 try:
-                    bundle.obj = self.obj_get(request, **kwargs)
+                    bundle.obj = self.obj_get(bundle=bundle, **kwargs)
                 except (queryset.DoesNotExist, exceptions.ObjectDoesNotExist):
                     raise tastypie_exceptions.NotFound("A document instance matching the provided arguments could not be found.")
 
@@ -785,12 +850,7 @@ class MongoEngineListResource(MongoEngineResource):
             if pk_field is None:
                 object_list[bundle.obj.pk] = bundle.obj
             else:
-                for i, obj in enumerate(object_list):
-                    if getattr(obj, pk_field) == bundle.obj.pk:
-                        object_list[i] = bundle.obj
-                        break
-                else:
-                    raise IndexError("Embedded document with primary key '%s' not found." % bundle.obj.pk)
+                object_list[self.find_embedded_document(object_list, pk_field, bundle.obj.pk)] = bundle.obj
 
             self.save_related(bundle)
 
@@ -802,12 +862,12 @@ class MongoEngineListResource(MongoEngineResource):
         except mongoengine.ValidationError, e:
             raise exceptions.ValidationError(e.message)
 
-    def obj_delete(self, request=None, **kwargs):
+    def obj_delete(self, bundle, **kwargs):
         obj = kwargs.pop('_obj', None)
 
         if not getattr(obj, 'pk', None):
             try:
-                obj = self.obj_get(request, **kwargs)
+                obj = self.obj_get(bundle=bundle, **kwargs)
             except (queryset.DoesNotExist, exceptions.ObjectDoesNotExist):
                 raise tastypie_exceptions.NotFound("A document instance matching the provided arguments could not be found.")
 
@@ -817,12 +877,7 @@ class MongoEngineListResource(MongoEngineResource):
         if pk_field is None:
             object_list.pop(obj.pk)
         else:
-            for i, obj in enumerate(object_list):
-                if getattr(obj, pk_field) == obj.pk:
-                    del object_list[i]
-                    break
-            else:
-                raise IndexError("Embedded document with primary key '%s' not found." % obj.pk)
+            object_list.pop(self.find_embedded_document(object_list, pk_field, obj.pk))
 
         # Make sure to delete FileField files
         for fieldname, field in obj._fields.items():
@@ -831,7 +886,7 @@ class MongoEngineListResource(MongoEngineResource):
 
         self.instance.save()
 
-    def get_resource_uri(self, bundle_or_obj):
+    def detail_uri_kwargs(self, bundle_or_obj):
         if isinstance(bundle_or_obj, tastypie_bundle.Bundle):
             obj = bundle_or_obj.obj
         else:
@@ -854,4 +909,13 @@ class MongoEngineListResource(MongoEngineResource):
         if self._meta.api_name is not None:
             kwargs['api_name'] = self._meta.api_name
 
-        return self._build_reverse_url('api_dispatch_subresource_detail', kwargs=kwargs)
+        return kwargs
+
+    def get_resource_uri(self, bundle_or_obj=None, url_name='api_dispatch_subresource_list'):
+        if bundle_or_obj is not None:
+            url_name = 'api_dispatch_subresource_detail'
+
+        try:
+            return self._build_reverse_url(url_name, kwargs=self.resource_uri_kwargs(bundle_or_obj))
+        except urlresolvers.NoReverseMatch:
+            return ''
