@@ -8,6 +8,10 @@ v1_api = api.Api(api_name='v1')
 v1_api.register(resources.PersonResource())
 v1_api.register(resources.PersonObjectClassResource())
 v1_api.register(resources.OnlySubtypePersonResource())
+v1_api.register(resources.IndividualResource())
+v1_api.register(resources.CompanyResource())
+v1_api.register(resources.ContactResource())
+v1_api.register(resources.ContactGroupResource())
 v1_api.register(resources.CustomerResource())
 v1_api.register(resources.BoardResource())
 v1_api.register(resources.DocumentWithIDResource())
@@ -22,6 +26,10 @@ v1_api.register(resources.ReferencedListFieldNonFullTestResource())
 v1_api.register(resources.BooleanMapTestResource())
 v1_api.register(resources.EmbeddedListWithFlagFieldTestResource())
 v1_api.register(resources.AutoAllocationFieldTestResource())
+v1_api.register(resources.ExporterResource())
+v1_api.register(resources.PipeResource())
+v1_api.register(resources.BlankableParentResource())
+v1_api.register(resources.ReadonlyParentResource())
 
 urlpatterns = patterns('',
     url(r'^api/', include(v1_api.urls)),
